SystemOrganization addCategory: #XPacMan!

Object subclass: #GameElement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPacMan'!

!GameElement class methodsFor: 'as yet unclassified' stamp: 'mw 6/9/2012 14:52'!
elementExtent
	"returns the with and height of every game element"
	^30@30! !

!GameElement methodsFor: 'drawing' stamp: 'mw 6/9/2012 15:35'!
position: loc
	| delta x1 x2 y1 y2 |
	delta := GameElement elementExtent.
	x1 := (loc x - 1) * delta x.
	x2 := x1 + delta x.
	y1 := (loc y - 1) * delta y.
	y2 := y1 + delta y.
	^Rectangle left: x1 right: x2 top: y1 bottom:y2! !

!GameElement methodsFor: 'drawing' stamp: 'mw 6/9/2012 15:42'!
render: form at: loc
	"should render the element to its position on the screen"
	| line line2 line3 line4 position |
	position := self position: loc.
	line := Line
		from: (position left)@(position top)
		to: (position right)@(position top)
		withForm: (Form extent: 1@1 depth: 8) fillBlack.
	line displayOn: form.
	line2 := Line
		from: (position right)@(position top)
		to: (position right)@(position bottom)
		withForm: (Form extent: 1@1 depth: 8) fillBlack.
	line2 displayOn: form.
	line3 := Line
		from: (position right)@(position bottom)
		to: (position left)@(position bottom)
		withForm: (Form extent: 1@1 depth: 8) fillBlack.
	line3 displayOn: form.
	line4 := Line
		from: (position left)@(position bottom)
		to: (position left)@(position top)
		withForm: (Form extent: 1@1 depth: 8) fillBlack.
	line4 displayOn: form! !

GameElement subclass: #Pawn
	instanceVariableNames: 'color'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPacMan'!

!Pawn methodsFor: 'accessing' stamp: 'mw 6/9/2012 16:13'!
color
	"Answer the value of color"

	^ color! !

!Pawn methodsFor: 'accessing' stamp: 'mw 6/9/2012 16:13'!
color: anObject
	"Set the value of color"

	color := anObject! !

!Pawn methodsFor: 'drawing' stamp: 'mw 6/9/2012 17:22'!
render: form at: loc
	| position fillForm circle offset radius |
	offset := GameElement elementExtent.
	radius := (offset x/2) - 2.
	position := self position: loc.
	circle := Circle new.
	fillForm := Form extent: 2@2 depth: 8.
	fillForm fillColor: self color.
	circle form: fillForm.
	circle radius: radius.
	circle center: (position left + (offset x / 2))@(position top + (offset y / 2)).
	circle displayOn: form.! !

GameElement subclass: #PointField
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPacMan'!

GameElement subclass: #WallField
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPacMan'!

!WallField methodsFor: 'as yet unclassified' stamp: 'mw 6/9/2012 15:44'!
render: form at: loc
	| line position |
	super render: form at: loc.
	position := self position: loc.
	line := Line
		from: (position left)@(position top)
		to: (position right)@(position bottom)
		withForm: (Form extent: 1@1 depth: 8) fillBlack.
	line displayOn: form! !

Object subclass: #PlayingField
	instanceVariableNames: 'fields numRows numCeils'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPacMan'!

!PlayingField class methodsFor: 'as yet unclassified' stamp: 'mw 6/9/2012 14:34'!
newOfSize: aPoint
	| model |
	model := self basicNew.
	model
		numRows: aPoint y;
		numCeils: aPoint x.
	model initialize.
	^model! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/9/2012 13:56'!
at: aPoint
	^self fields at: aPoint! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/9/2012 13:57'!
at: aPoint put: aField
	self fields at: aPoint put: aField! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/3/2012 18:20'!
fields
	"Answer the value of fields"

	^ fields! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/3/2012 18:20'!
fields: anObject
	"Set the value of fields"

	fields := anObject! !

!PlayingField methodsFor: 'initialize-release' stamp: 'mw 6/3/2012 18:20'!
initialize
	super initialize.
	self initializeFields.! !

!PlayingField methodsFor: 'initialize-release' stamp: 'mw 6/3/2012 18:23'!
initializeFields
	self fields: Dictionary new.
	1 to: self numCeils do: [:x |
		1 to: self numRows do: [:y |
			| pt field |
			pt := x@y.
			field := GameElement new.
			self fields at: pt put: field]]! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/3/2012 18:24'!
numCeils
	"Answer the value of numCeils"
	numCeils isNil ifTrue: [self numCeils: 10].
	^ numCeils! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/3/2012 18:20'!
numCeils: anObject
	"Set the value of numCeils"

	numCeils := anObject! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/3/2012 18:25'!
numRows
	"Answer the value of numRows"
	numRows isNil ifTrue: [self numRows: 10].
	^ numRows! !

!PlayingField methodsFor: 'accessing' stamp: 'mw 6/3/2012 18:20'!
numRows: anObject
	"Set the value of numRows"

	numRows := anObject! !

!PlayingField methodsFor: 'drawing' stamp: 'mw 6/9/2012 15:19'!
render: form
	"should render the PlayingField and it child elements to the form"
	| elemLoc elem |
	1 to: numCeils do: [:y |
		1 to: numRows do: [:x |
			elemLoc := x@y.
			elem := self at: elemLoc.
			elem render: form at: elemLoc]].! !

Morph subclass: #PacGame
	instanceVariableNames: 'playingField fieldForm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XPacMan'!

!PacGame class methodsFor: 'as yet unclassified' stamp: 'mw 6/9/2012 17:19'!
createPlayingField
	| field player1 |
	field := PlayingField newOfSize: 10@10.
	field at: 1@1 put: WallField new.
	field at: 2@1 put: WallField new.
	field at: 3@1 put: WallField new.
	field at: 4@1 put: WallField new.
	field at: 5@1 put: WallField new.
	field at: 6@1 put: WallField new.
	field at: 7@1 put: WallField new.
	field at: 8@1 put: WallField new.
	field at: 9@1 put: WallField new.
	field at: 10@1 put: WallField new.
	
	player1 := Pawn new.
	player1 color: Color green.
	field at: 4@4 put: player1.
	^field! !

!PacGame class methodsFor: 'as yet unclassified' stamp: 'mw 6/9/2012 14:57'!
fieldExtendFor: aPlayingField
	"calculates the required size for a form containint aPlayingField"
	| x y |
	x := GameElement elementExtent x * aPlayingField numCeils.
	y := GameElement elementExtent y * aPlayingField numRows.
	^x@y! !

!PacGame methodsFor: 'drawing' stamp: 'mw 6/9/2012 17:44'!
calculateExtent
	| pt |
	pt := self fieldForm extent.
	pt := pt + (self panelWidth@0).
	^pt! !

!PacGame methodsFor: 'accessing' stamp: 'mw 6/9/2012 17:40'!
fieldForm
	"Answer the value of fieldForm"

	^ fieldForm! !

!PacGame methodsFor: 'accessing' stamp: 'mw 6/9/2012 17:40'!
fieldForm: anObject
	"Set the value of fieldForm"

	fieldForm := anObject! !

!PacGame methodsFor: 'initialize-release' stamp: 'mw 6/9/2012 17:45'!
initialize
	super initialize.
	self playingField: PacGame createPlayingField.
	self fieldForm: (Form extent: (self class fieldExtendFor: self playingField) depth: Display depth).
	self fieldForm fillColor: Color white.
	self setExtent.
	self setupMorphs.
	! !

!PacGame methodsFor: 'drawing' stamp: 'mw 6/9/2012 17:48'!
makeControlPanelMorph
	| panel |
	panel := RectangleMorph new borderWidth: 0;
			color: Color white;
			layoutPolicy: ProportionalLayout new.
	^panel! !

!PacGame methodsFor: 'drawing' stamp: 'mw 6/9/2012 17:44'!
panelWidth
	^100
! !

!PacGame methodsFor: 'accessing' stamp: 'mw 6/9/2012 17:26'!
playingField
	"Answer the value of playingField"

	^ playingField! !

!PacGame methodsFor: 'accessing' stamp: 'mw 6/9/2012 17:26'!
playingField: anObject
	"Set the value of playingField"

	playingField := anObject! !

!PacGame methodsFor: 'drawing' stamp: 'mw 6/9/2012 17:45'!
setExtent
	self extent: self calculateExtent! !

!PacGame methodsFor: 'initialize-release' stamp: 'mw 6/9/2012 17:47'!
setupMorphs
	self layoutPolicy: ProportionalLayout new.
	self
		addMorph: self makeGameFieldMorph
		fullFrame: (LayoutFrame
				fractions: (0 @ 0 corner: 1 @ 1)
				offsets: (0 @ 0 corner: self panelWidth negated @ 0)).
	self
		addMorph: self makeControlPanelMorph
		fullFrame: (LayoutFrame
				fractions: (0 @ 0 corner: 1 @ 1)
				offsets: (self panelWidth negated @ 0 corner: 0 @ 0)).! !
